/*===========================================================================*/
/*                                                                           */
/* File    : CALC.C                                                          */
/*                                                                           */
/* Purpose : Demo of the MS-Windows calculator.                              */
/*                                                                           */
/* History :                                                                 */
/*                                                                           */
/* (C) Copyright 1989 Marc Adler/Magma Systems     All Rights Reserved       */
/*===========================================================================*/

#include <ctype.h>
#include <math.h>
#include <string.h>
#include "window.h"
#include "keys.h"
#include "rc.h"


#define DLG_CALC      200
#define ID_DISPLAY    110
#define ID_MC         111
#define ID_MR         112
#define ID_MP         113
#define ID_MM         114

#include "desktop.h"


HWND hMain;
HWND hWndCalc = NULLHWND;
static HWND hWndLastFocusItem = NULLHWND;

#ifdef DESKTOP
extern HWND hWndDir;
#endif

extern int pascal MainWndProc();
extern int pascal CalcDialogDriver(WORD idDlg, int (pascal *pfnDlg)());
extern int pascal CalcDialogProc();
extern char *Calculate(char *s1, char *s2, int op);
extern char *ZeroRightTrim(char *szNum);


#ifndef DESKTOP
main(argc, argv)
  int  argc;
  char **argv;
{
  EVENT  event;
  WNDCLASS wndClass;
  WORD   hAccel;

  /*
    Initialize the window system - this *must* be done once in every appl
  */
  WinInit();
  WinUseSysColors(NULLHWND, TRUE);

  /*
    Create the main window and set the main window procedure
  */
  hMain = CreateWindow("Normal",       /* class */
                       "DESKTOP",      /* title */
                       WIN_TITLEBAR | WIN_SYSMENU | WIN_MINMAXBOX |
                       WIN_HAS_BORDER,
                       0,0, 80,25,     /* x,y,width,height */
                       SYSTEM_COLOR,   /* attr */
                       0,              /* id */
                       NULLHWND,       /* hParent */
                       NULLHWND,       /* hMenu */
                       0,              /* hInst */
                       0L);            /* lpParam */
  WinSetWinProc(hMain, MainWndProc);
  ShowWindow(hMain, TRUE);

  hWndCalc = (HWND) CalcDialogDriver(DLG_CALC, CalcDialogProc);
#if MODELESS
  if (hWndCalc)
    ShowWindow(hWndCalc);
#endif

  /*
    Main message loop
  */
  while (GetMessage(&event) != WM_QUIT)
  {
#if MODELESS
    if (hWndCalc && IsDialogMessage(hWndCalc, &event))
      continue;
#endif

    TranslateAccelerator(&event);
    DispatchMessage(&event);
  }

  exit(0);
}
#endif


#ifndef DESKTOP
/*
  MainWndProc - window procedure for the main window
*/
int pascal MainWndProc(hWnd, message, wParam, lParam)
  HWND hWnd;
  WORD message;
  WORD wParam;
  DWORD lParam;
{
  switch (message)
  {
    case WM_SYSCOMMAND :
      switch (wParam)
      {
        case SC_CLOSE :
          PostQuitMessage(0);
          break;
        default :
          return DefWinProc(hWnd, message, wParam, lParam);
      }
      break;

    default :
      /* Call the default window procedure for the main window */
      return DefWinProc(hWnd, message, wParam, lParam);
  }
}
#endif


Calculator()
{
#if MODELESS
  if (hWndCalc)
  {
    /*
      It's active, but hidden...
    */
    BringWindowToTop(hWndCalc);
    ShowWindow(hWndCalc, TRUE);
    SetFocus(hWndLastFocusItem);
    return TRUE;
  }
#endif

  hWndCalc = (HWND) CalcDialogDriver(DLG_CALC, CalcDialogProc);
#if MODELESS
  if (hWndCalc)
  {
    ShowWindow(hWndCalc, TRUE);
    SetFocus(hWndLastFocusItem = GetDlgItem(hWndCalc, IDCANCEL));
  }
#endif
}


int pascal CalcDialogDriver(WORD idDlg, int (pascal *pfnDlg)())
{
  WORD hModule;
  HDLG hDlg;

  if ((hModule = OpenResourceFile(CALC_RESFILE)) == FALSE)
  {
    MessageBox("Can't open CALC.RES", NULL, NULL, "Error", MB_OK);
    return FALSE;
  }
#ifdef DESKTOP
  if ((hDlg = LoadDialog(hModule, idDlg, hWndDir, (PFBOOL) pfnDlg)) == NULLHWND)
#else
  if ((hDlg = LoadDialog(hModule, idDlg, hMain, (PFBOOL) pfnDlg)) == NULLHWND)
#endif
  {
    MessageBox("Can't load the dialog box", NULL, NULL, "Error", MB_OK);
    return FALSE;
  }
  CloseResourceFile(hModule);

#if MODELESS
  WinSetFlags(hDlg, WS_CLIP | WIN_TITLEBAR | WIN_MOVEBOX);
  CalcDialogProc(hDlg, WM_INITDIALOG, 0, 0L);
  return (int) hDlg;
#else
  return DialogBox(hDlg);
#endif
}


char szTotal[80];
char szNumber[80];
char szMemory[80];


int pascal CalcDialogProc(hDlg, message, wParam, lParam)
  HWND hDlg;
  WORD message;
  WORD wParam;
  DWORD lParam;
{
  static BOOL bNewNumber  = TRUE;
  static BOOL bMemoryOn   = FALSE;
  static BOOL bGotM       = FALSE;
  static BOOL bPressedDec = FALSE;
  static int  PendingOp   = '=';
  char        *pDecimal;

  switch (message)
  {
    case WM_INITDIALOG :
      PendingOp   = '=';
      bNewNumber  = TRUE;
      bMemoryOn   = FALSE;
      bGotM       = FALSE;
      bPressedDec = FALSE;

      strcpy(szTotal,  "0.");
      strcpy(szNumber, "0.");
      strcpy(szMemory, "0.");

      DisplayNumber(hDlg, szTotal, bMemoryOn);
      return TRUE;

#if MODELESS
#if 0
    case WM_LBUTTONDOWN:
    {
      RECT rWindow;
      int  mouserow = HIWORD(lParam);
      int  mousecol = LOWORD(lParam);

      GetWindowRect(hDlg, &rWindow);
      if (mouserow == rWindow.top)
      {
        Rubberband(hDlg, TRUE);
        ShowWindow(hDlg, TRUE);
        return TRUE;
      }
      return FALSE;
    }
#endif
#endif


    case WM_CHAR       :
      switch (toupper(wParam))
      {
        case VK_TAB     :
        case VK_BACKTAB :
          return FALSE;

        case VK_BACKSPACE :
          if ((pDecimal = strchr(szNumber, '.')) != NULL)
          {
            if (pDecimal[1] == '\0')
            {
              /* We have a whole number */
              if (!strcmp(szNumber, "0."))
                return TRUE;         /* don't do anything to "0." */
              if (pDecimal == szNumber + 1)
                szNumber[0] = '0';   /* change "4." to "0." */
              else
              {
                *pDecimal-- = '\0';
                *pDecimal = '.';     /* change 43. to 4. */
              }
            }
            else
            {
              /* We have a real number */
              pDecimal[1] = '\0';
            }
            DisplayNumber(hDlg, szNumber, bMemoryOn);
          }
          return TRUE;

        case VK_ESC       :
          wParam = 'C';
          break;

        case VK_RETURN    :
          wParam = '=';
          break;

        case 'N'          :
          wParam = '';
          break;

        case 'Q'          :
          wParam = IDCANCEL;
          break;

        case 'S'          :
          wParam = '';
          break;

        case 'M'          :
          bGotM = TRUE;
          return TRUE;

        case 'R'          :
          if (bGotM)
          {
            bGotM = FALSE;
            wParam = ID_MR;
            break;
          }
          return TRUE;

        case '+'          :
        case '-'          :
          if (bGotM)
          {
            bGotM = FALSE;
            wParam = (wParam == '+') ? ID_MP : ID_MM;
          }
          break;

      }
      /* fall through ... */


    case WM_COMMAND    :
      switch (wParam)
      {
        case IDCANCEL :
#if MODELESS
          DestroyWindow(hWndCalc);
#if DESKTOP
          SetFocus(hWndDir);
#endif
          hWndCalc = NULLHWND;
#else
          EndDialog(hDlg, TRUE);
#endif
          break;

        case '0' :   case '1' :   case '2' :   case '3' :
        case '4' :   case '5' :   case '6' :   case '7' :
        case '8' :   case '9' :
        case ID_MR:
          if (bNewNumber)
          {
            strcpy(szTotal, szNumber);
            strcpy(szNumber, "0.");
            bNewNumber = FALSE;
            bPressedDec = FALSE;
          }

          if (wParam == ID_MR)
            strcpy(szNumber, szMemory);
          else
          {
            char szDigit[2];
            szDigit[0] = (char) wParam;  szDigit[1] = '\0';
            pDecimal = strchr(szNumber, '.');
            if (bPressedDec)
            {
              /* Append the number to the end */
              strcat(szNumber, szDigit);
            }
            else if (!strcmp(szNumber, "0."))
              szNumber[0] = (char) wParam;
            else
            {
              /* Insert the number before the decimal pt */
              pDecimal[0] = (char) wParam;
              pDecimal[1] = '.';
              pDecimal[2] = '\0';
            }
          }

          DisplayNumber(hDlg, szNumber, bMemoryOn);
          break;

        case '.'          :
          bPressedDec = TRUE;
          break;

        case ID_MC :
          strcpy(szMemory, "0.");
          DisplayNumber(hDlg, szNumber, bMemoryOn = FALSE);
          break;

        case ID_MM :
        case ID_MP :
          bNewNumber  = TRUE;
          bGotM       = FALSE;
          bPressedDec = FALSE;
          Calculate(szMemory, szNumber, wParam == ID_MP ? '+' : '-');
          DisplayNumber(hDlg, szNumber, bMemoryOn = TRUE);
          break;

        case 'C'   :
          PendingOp   = '=';
          bNewNumber  = TRUE;
          bGotM       = FALSE;
          bPressedDec = FALSE;
          strcpy(szTotal,  "0.");
          strcpy(szNumber, "0.");
          DisplayNumber(hDlg, szNumber, bMemoryOn);
          return TRUE;

        default  :
          if (wParam == '' || wParam == '')
          {
            Calculate(szNumber, szNumber, wParam);
            DisplayNumber(hDlg, szNumber, bMemoryOn);
          }
          else if (!bNewNumber)
          {
            Calculate(szTotal, szNumber, PendingOp);
            DisplayNumber(hDlg, szTotal, bMemoryOn);
          }
          bNewNumber  = TRUE;
          bGotM       = FALSE;
          bPressedDec = FALSE;
          PendingOp   = wParam;
          break;
      } /* switch (wParam) */

      SetFocus(hWndLastFocusItem = GetDlgItem(hDlg, wParam));
      return TRUE;
  }

  /*
    Return FALSE if you did not process the message. This gives the default
    dialog proc a crack at handling it. Note - do NOT call any DefWindowProc
    here; returning FALSE does the same thing.
  */
  return FALSE;
}


char *Calculate(char *s1, char *s2, int op)
{
  double n1, n2;

  n1 = atof(s1);
  n2 = atof(s2);

  switch (op)
  {
    case '+'  : n1 += n2;
                break;
    case '-'  : n1 -= n2;
                break;
    case '*'  : n1 *= n2;
                break;
    case '/'  : n1 = (n1) ? n1 / n2 : 0;
                break;
    case '='  : n1 = n2;
                break;
    case ''  : n1 = -n2;
                break;
    case '%'  : n1 *= (n2 * 0.01);
                break;
    case ''  : n1 = sqrt(n2);
                break;
    case 'C'  :
    default   : n1 = 0;
                break;
  }

  sprintf(s1, (n1 == 0) ? "0." : "%lf", n1);
  return ZeroRightTrim(s1);
}


DisplayNumber(HDLG hDlg, char *szNum, BOOL bMem)
{
  char buf[80];
  sprintf(buf, "%c%-27s", (bMem) ? 'M' : ' ', szNum);
  SetDlgItemText(hDlg, ID_DISPLAY, buf);
}


char *ZeroRightTrim(char *szNum)
{
  char *s;

  if ((s = strchr(szNum, '.')) != NULL)
    for (s = szNum + strlen(szNum) - 1;  *s == '0';  *s-- = '\0')
      ;
  return szNum;
}

